;---------------------------------------------------------------------
;                   Test GUS16 V6 source code
;                       Jesús Arias (2023)
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; Programming Conventions:
; Strict:
; - Core revision 6
; - R7 is the Stack Pointer	(software)
; - R6 holds the return address for subroutines (core parameter)
; - Full-descending Stack
; Loose guidelines:
; - R0, R1, R2 are used for arguments / results
;              Values may change on subroutine calls
; - R3, R4, R5 are used for local variables
;              Values are preserved on subroutine calls
;---------------------------------------------------------------------
;---------------------------------------------------------------------
;----------- I/O MAP  -----------

IOBASE=		0x20
IRQEN=		0x20
PFLAGS=		0x21
UARTDAT=	0x22

; Offsets relative to IOBASE
OIRQEN=		0
OPFLAGS=	1
OUARTDAT=	2

;--------- Interrupts --------
IRQ0EN=		1
;--------- PFLAGS register bits -----------
UARTDV=		1		; Data valid (UART RX)
UARTRDY=	2		; Transmitter busy (UART TX)


;--------------------------------------
;----------- VECTORS -----------------
;-------------------------------------- 

		org 0			; RESET
inicio:	jr		start

		org		0x4		; IRQ 0
;--------------------------------------
;-----------   Interrupts -------------
;-------------------------------------- 


irq0:	subi	r7,3
		st		(r7+0),r0
		st		(r7+1),r1
		st		(r7+2),r2
		
		ldi		r2,nxtchar
		ldi		r1,IOBASE
		ld		r0,(r2)
		st		(r1+OUARTDAT),r0
		addi	r0,1
		cmpi	r0,128
		jnz		ir2
		ldi		r0,' '
ir2:	st		(r2),r0

ir9:	ld		r0,(r7+0)			; return from interrupt
		ld		r1,(r7+1)
		ld		r2,(r7+2)
		addi	r7,3
		reti

;--------------------------------------
;----------- VARIABLES (near) ---------
;-------------------------------------- 

		org	0x40		
nxtchar:
		word 32
		

;--------------------------------------
;-----------   MAIN   -----------------
;-------------------------------------- 

start:

		ldpc	r7			; stack at the end of memory
		word	stack	
		
		;ldi		r1,IOBASE	; enable UART TX IRQ
		;ldi		r0,1
		;st		(r1),r0	

		ldpc	r0
		word	txtbe
		jal		putsbe

;lab100:	jal 	getch
;		jal		putch
;		jr		lab100
		
		ldi		r4,0
		ldi		r5,'\n'		
lab20:	or		r0,r4,r4
		jal		prthex
		ldi		r0,0
		ldi		r1,95
		jal		delay
		addi	r4,1
		jr		lab20
		
lab0:	ldi		r0,' '
		ldi		r1,IOBASE
		
lab1:	ld		r2,(r1+OPFLAGS)
		andi	r2,2
		jnz		lab1
		st		(r1+OUARTDAT),r0
		addi	r0,1
		cmpi	r0,128
		jnz		lab1
		jr		lab0
		
;----------------------------------------------
; Send data to UART
; Parameters:	R0: data to send
; Returns:		none
; Modiffies:	none

putch:	subi	r7,1
		st		(r7),r6
ptch1:	ldi		r6,IOBASE		; wait while TX busy
		ld		r6,(r6+OPFLAGS)		
		andi	r6,UARTRDY
		jz		ptch1
		ldi		r6,IOBASE		
		st		(r6+OUARTDAT),r0	; send data
		ld		r6,(r7)
		addi	r7,1
		jind	r6		


;----------------------------------------------
; Get data from UART (blocking)
; Parameters:	none
; Returns:		R0: data received
; Modiffies:	none

getch:	ldi		r0,IOBASE	
		ld		r0,(r0+OPFLAGS)	; wait until data valid
		andi	r0,UARTDV
		jz		getch
		ldi		r0,IOBASE
		ld		r0,(r0+OUARTDAT)	; get the data
		jind	r6

;-----------------------------------------------
;		print hexadecimal number
;-----------------------------------------------
; Parameters:	R0:	data to print
; Returns:		none
; Modiffies:	no register

prthex:	subi	r7,5
		st		(r7+0),r0
		st		(r7+1),r1
		st		(r7+2),r4
		st		(r7+3),r5
		st		(r7+4),r6
		or		r4,r0,r0
		ldi		r5,4
phl1:	rori	r4,r4,12
		or		r0,r4,r4
		andi	r0,15
		cmpi	r0,10
		jnc		phl2
		addi	r0,7
phl2:	addi	r0,'0'
		jal		putch
		subi	r5,1
		jnz		phl1

		ld		r0,(r7+3)	; R5
		jal		putch
		
		ld		r0,(r7+0)
		ld		r1,(r7+1)
		ld		r4,(r7+2)
		ld		r5,(r7+3)
		ld		r6,(r7+4)
		addi	r7,5
		jind	r6
		
putsbe:	subi	r7,3
		st		(r7+0),r0
		st		(r7+1),r1
		st		(r7+2),r6
		
		or		r1,r0,r0
ptsb1:	ld		r0,(r1)
		rori	r0,r0,8
		andi	r0,255
		jz		ptsb9
		jal		putch
		ld		r0,(r1)
		andi	r0,255
		jz		ptsb9		
		jal		putch
		addi	r1,1
		jr		ptsb1
		
ptsb9:	ld		r0,(r7+0)
		ld		r1,(r7+1)
		ld		r6,(r7+2)
		addi	r7,3
		jind	r6
		
delay:	subi	r0,1
		sbci	r1,0
		jc		delay
		jind	r6
		
txtbe:	asczbe	"\nGUS16 v6 in cordia (J. Arias 2023)\n"

;-------- End of code
endcode:

;-------- end of memory -------
		org	4*1024
stack:

